/**
 * @file
 * @brief   Demonstrates a watchdog timer in run mode
 *
 * @details When the program starts, LED1 blinks three times and stops.
 *          Open a terminal program to see interrupt messages.
 *
 *          SW2: Push SW2 to trigger a watchdog reset. This will reset the
 *               watchdog before the wait period has expired and trigger an
 *               interrupt.
 *
 *          SW3: Push SW3 to trigger a delay and see the program restart by
 *               blinking LED1 three times. This delay is long enough for the
 *               reset period to expire triggering a reset.
 */

/* *****************************************************************************
 * Copyright (C) 2016 Maxim Integrated Products, Inc., All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated
 * Products, Inc. shall not be used except as stated in the Maxim Integrated
 * Products, Inc. Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products, Inc. retains all
 * ownership rights.
 *
 * $Date: 2016-03-11 11:46:02 -0600 (Fri, 11 Mar 2016) $
 * $Revision: 21838 $
 *
 **************************************************************************** */

/* **** Includes **** */
#include <stdio.h>
#include <stdint.h>
#include "mxc_config.h"
#include "nvic_table.h"
#include "board.h"
#include "mxc_sys.h"
#include "wdt.h"
#include "tmr_utils.h"
#include "led.h"

/* **** Definitions **** */
/* **** Globals **** */
//use push buttons defined in board.h
extern const gpio_cfg_t pb_pin[];

// refers to array, do not change constants
#define SW2 0
#define SW3 1

/* **** Functions **** */

/* ************************************************************************** */
void watchdog_timeout_handler()
{
    //get and clear flag
    WDT_ClearIntFlag(MXC_WDT0);
    printf("TIMEOUT! \n");
}

/* ************************************************************************** */
void WDT0_IRQHandler(void)
{
	watchdog_timeout_handler();
}
/* ************************************************************************** */
void WDT_Setup()
{
    WDT_Enable(MXC_WDT0, 0);
    WDT_Enable(MXC_WDT0, 1);
}

/* ************************************************************************** */
int main(void)
{
    const sys_cfg_tmr_t sys_tmr_cfg = NULL; /* No system specific configuration needed. */

	if (WDT_GetResetFlag(MXC_WDT0)) {
	        WDT_ClearResetFlag(MXC_WDT0);
	        WDT_EnableReset(MXC_WDT0, 0);
	        WDT_Enable(MXC_WDT0, 0);
	        printf("Watchdog reset\n");
	}

    printf("\n************** Watchdog Timer Demo ****************\n");
    printf("Press a button to create watchdog interrupt or reset:\n");
    printf("SW2 = reset program\n");
    printf("SW3 = timeout interrupt\n");
    LED_Off(0);

    //blink LED1 three times at startup
    int numBlinks = 3;

    while (numBlinks) {
        LED_On(1);
        TMR_Delay(MXC_TMR0, MSEC(100), &sys_tmr_cfg);
        LED_Off(1);
        TMR_Delay(MXC_TMR0, MSEC(100), &sys_tmr_cfg);
        numBlinks--;
    }

    //setup watchdog
    WDT_Setup();

    while (1) {

        //Push SW2 to reset watchdog
        if (GPIO_InGet(&pb_pin[SW2]) == 0) {
		printf("Program Reset\n");
            WDT_SetResetPeriod(MXC_WDT0, WDT_PERIOD_2_18);
            WDT_EnableReset(MXC_WDT0, 1);
            WDT_EnableInt(MXC_WDT0, 0);
            WDT_ResetTimer(MXC_WDT0);
            while(1);
        }

        //Push SW3 to start longer delay - shows Interrupt before the reset happens
        if (GPIO_InGet(&pb_pin[SW3]) == 0) {
            printf("delaying...\n");
            WDT_SetResetPeriod(MXC_WDT0, WDT_PERIOD_2_27);
            WDT_SetIntPeriod(MXC_WDT0, WDT_PERIOD_2_26);
            WDT_EnableReset(MXC_WDT0, 1);
        	WDT_EnableInt(MXC_WDT0, 1);
            NVIC_EnableIRQ(WDT0_IRQn);
            while(1);
        }

        //blink LED0
        TMR_Delay(MXC_TMR0, MSEC(100), &sys_tmr_cfg);
        LED_On(0);
        TMR_Delay(MXC_TMR0, MSEC(100), &sys_tmr_cfg);
        LED_Off(0);

        //Reset watchdog
        WDT_ResetTimer(MXC_WDT0);
    }
}
